@@ -13,7 +13,7 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: fileName.c,v 1.7 1996/06/09 17:27:49 torsten Exp $ */
+/* $Id: fileName.c,v 1.10 1998/02/24 20:38:43 torsten Exp $ */
 
 #include <X11/StringDefs.h>
 #include <X11/Intrinsic.h>
@@ -37,7 +37,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#if defined(SYSV) || defined(SVR4)
+#if defined(SYSV) || defined(SVR4) || defined(__CYGWIN32__)
 #include <dirent.h>
 #else
 #include <sys/dir.h>
@@ -375,7 +375,7 @@ static void
 setCWD(arg_t * arg, char *dir)
 {
     DIR *dirp;
-#if defined(SYSV) || defined(SVR4) || defined(__alpha)
+#if defined(SYSV) || defined(SVR4) || defined(__alpha) || defined(__CYGWIN32__)
     struct dirent *e;
 #else
     struct direct *e;
@@ -439,7 +439,11 @@ setCWD(arg_t * arg, char *dir)
 	/*
 	**  Now only if it is a file.
 	 */
+#ifdef  __EMX__
+	{
+#else
 	if ((statbuf.st_mode & (S_IFMT & ~S_IFLNK)) == 0) {
+#endif
 	    list[i++] = XtNewString(nm);
 	    fileCount++;
 	}
@@ -686,10 +690,13 @@ buildOpenBroswer(Widget w, arg_t * arg)
     XtTranslations toglt;
     int i;
     char **list;
-
-    shell = XtVaCreatePopupShell("filebrowser",
-			       transientShellWidgetClass, GetToplevel(w),
-				 NULL);
+    Arg args[3];
+    int nargs = 0;
+    
+    
+    shell = XtVisCreatePopupShell("filebrowser",
+				  transientShellWidgetClass, GetToplevel(w),
+				  args, nargs);
     form = XtVaCreateManagedWidget("form",
 				   formWidgetClass, shell,
 				   XtNborderWidth, 0,
@@ -766,10 +773,12 @@ buildSaveBroswer(Widget w, arg_t * arg)
     XtTranslations toglt;
     int i;
     char **list, *rdr = NULL;
-
-    shell = XtVaCreatePopupShell("filebrowser",
+    Arg args[3];
+    int nargs=0;
+    
+    shell = XtVisCreatePopupShell("filebrowser",
 			       transientShellWidgetClass, GetToplevel(w),
-				 NULL);
+				args, nargs);
     form = XtVaCreateManagedWidget("form",
 				   formWidgetClass, shell,
 				   XtNborderWidth, 0,
@@ -838,14 +847,16 @@ buildSaveBroswer(Widget w, arg_t * arg)
 }
 
 static Widget
-buildSimpleBroswer(Widget w, arg_t * arg, Boolean isSave)
+buildSimpleBrowser(Widget w, arg_t * arg, Boolean isSave)
 {
     Widget shell, browser, okButton, cancelButton, form;
     XtAccelerators accel;
+    Arg args[3];
+    int nargs = 0;
 
-    shell = XtVaCreatePopupShell("filebrowser",
-			       transientShellWidgetClass, GetToplevel(w),
-				 NULL);
+    shell = XtVisCreatePopupShell("filebrowser",
+				  transientShellWidgetClass, GetToplevel(w),
+				  args, nargs);
     form = XtVaCreateManagedWidget("form",
 				   formWidgetClass, shell,
 				   XtNborderWidth, 0,
@@ -949,10 +960,10 @@ getArg(Widget w, Boolean type, Boolean * built)
 	shell = buildSaveBroswer(w, cur);
 	break;
     case 2:
-	shell = buildSimpleBroswer(w, cur, True);
+	shell = buildSimpleBrowser(w, cur, True);
 	break;
     case 3:
-	shell = buildSimpleBroswer(w, cur, False);
+	shell = buildSimpleBrowser(w, cur, False);
 	break;
     }
 
