@@ -13,7 +13,7 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: iprocess.c,v 1.11 1996/05/09 07:12:23 torsten Exp $ */
+/* $Id: iprocess.c,v 1.12 1998/04/16 19:42:01 torsten Exp $ */
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -56,10 +56,18 @@ convolve(Image * input, float *mat, int n,
     output = ImageNew(input->width, input->height);
     op = output->data;
 
+    sum = 0;
+    for (yy = 0; yy < n; yy++) {
+	for (xx = 0; xx < n; xx++) {
+	    sum += mat[xx + n * yy];
+	}
+    }
+    if (sum <= 0)
+	sum = 1;
+
     for (y = 0; y < input->height; y++) {
 	for (x = 0; x < input->width; x++) {
 	    r = g = b = 0;
-	    sum = 0;
 	    for (yy = 0; yy < n; yy++) {
 		for (xx = 0; xx < n; xx++) {
 		    xv = x + xx - 1;
@@ -72,11 +80,8 @@ convolve(Image * input, float *mat, int n,
 		    r += (float) *p++ * mat[xx + n * yy];
 		    g += (float) *p++ * mat[xx + n * yy];
 		    b += (float) *p * mat[xx + n * yy];
-		    sum += mat[xx + n * yy];
 		}
 	    }
-	    if (sum <= 0)
-		sum = 1;
 	    if (absFlag) {
 		if (r < 0)
 		    r = -r;
@@ -235,15 +240,33 @@ ImageEmbose(Image * input)
 Image *
 ImageInvert(Image * input)
 {
-    Image *output = ImageNewCmap(input->width, input->height, input->cmapSize);
+    Image *output;
     int i;
     unsigned char *ip, *op;
     int count;
+    int x,y;
+    unsigned char *p;
 
+    output = ImageNew(input->width, input->height);
+    op = output->data;
+
+    for (y = 0; y < input->height; y++)
+    {
+	for (x = 0; x < input->width; x++)
+	{
+	    p = ImagePixel(input, x, y);
+	    *op++ = 255 - *p++;
+	    *op++ = 255 - *p++;
+	    *op++ = 255 - *p;
+	}
+    }
+
+#if 0
     /*
     **	 If the input has a colormap, just invert that.
      */
-    if (input->cmapSize != 0) {
+    if (input->scale == 1 && input->cmapSize != 0) {
+	output = ImageNewCmap(input->width, input->height, input->cmapSize);
 	ip = input->cmapData;
 	op = output->cmapData;
 	count = input->cmapSize;
@@ -251,6 +274,7 @@ ImageInvert(Image * input)
 	memcpy(output->data, input->data,
 	     sizeof(char) * input->scale * input->width * input->height);
     } else {
+	output = ImageNew(input->width, input->height);
 	ip = input->data;
 	op = output->data;
 	count = input->width * input->height;
@@ -261,6 +285,7 @@ ImageInvert(Image * input)
 	*op++ = 255 - *ip++;
 	*op++ = 255 - *ip++;
     }
+#endif
 
     return output;
 }
