@@ -13,7 +13,7 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: main.c,v 1.15 1997/01/26 20:45:07 torsten Exp $ */
+/* $Id: main.c,v 1.17 1998/06/09 21:05:36 torsten Exp $ */
 
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
@@ -84,6 +84,8 @@ static XrmOptionDescRec options[] =
     {"-nowarn", ".nowarn", XrmoptionNoArg, (XtPointer) "True"},
     {"-help", ".help", XrmoptionNoArg, (XtPointer) "command"},
     {"+help", ".help", XrmoptionSepArg, (XtPointer) NULL},
+    {"-8", ".visualType", XrmoptionNoArg, (XtPointer) "cmap8"},
+    {"-visual", ".visualType", XrmoptionSepArg, (XtPointer) NULL},
 };
 
 
@@ -196,6 +198,82 @@ workProc(LocalInfo * l)
 	return FALSE;
 }
 
+/*
+ * You can specify:
+ * the visual
+ * the visual and the depth
+ * the visualID
+ * Any other combinations may not generate the results that you expect
+ * Also note that the visualID is strongly dependent on the platform
+ * being used.  We will let the user specify any visualID that they want.
+ */
+XVisualInfo
+GetFutureVisual(Widget *toplevel, int desiredDepth, int desiredVisual,
+		int desiredVisualID, Boolean *valid)
+{
+    Display *display = XtDisplay(*toplevel);
+    Colormap colormap; 
+    Window window; 
+    XSetWindowAttributes attributes; 
+    unsigned long valuemask; 
+    int screen_num = 0;
+    int i, visnum = 0;
+	
+    XVisualInfo rtnval;
+	
+    int visualsMatched;    
+     
+    XVisualInfo vTemplate;   
+    XVisualInfo visual[50]; /* If a machine has more than 50 visuals then this will fail. Higher end SGI's have 20 so this is probably OK */
+    XVisualInfo *visualList = visual;
+	
+
+    *valid=FALSE;
+      
+    vTemplate.screen = DefaultScreen(display);
+
+    if (desiredVisualID >= 0)
+        visualList = XGetVisualInfo (display, VisualScreenMask, &vTemplate, &visualsMatched);
+    else {
+        vTemplate.class = desiredVisual;
+	if (desiredDepth > 0) {
+	    vTemplate.depth = desiredDepth;
+	    visualList = XGetVisualInfo(display,
+					VisualClassMask | VisualScreenMask |
+					VisualDepthMask, &vTemplate,
+					&visualsMatched);
+	}
+	else {
+  	    visualList = XGetVisualInfo(display,
+					VisualClassMask | VisualScreenMask,
+					&vTemplate, &visualsMatched);
+	}
+    }
+
+    if (visualsMatched && desiredVisualID >= 0) {
+        visnum = -1;
+	/* look through the entire list for our visual ID */
+	for (i = 0; i < visualsMatched; i++) {
+  	    if (XVisualIDFromVisual(visualList[i].visual) == desiredVisualID)
+	        visnum = i;
+	}
+	if (visnum < 0)
+	    visualsMatched=0;
+    }
+	
+    if (visualsMatched == 0) {
+        fprintf(stderr, "No match found for visual\n");
+	exit(1);
+    }
+	
+    rtnval = visualList[visnum];
+    *valid = TRUE;
+	  
+    XFree(visualList);
+	
+    return rtnval;
+}
+
 
 void 
 main(int argc, char *argv[])
@@ -211,11 +289,149 @@ main(int argc, char *argv[])
     XrmDatabase rdb;
     Boolean isIcon;
 
-    InitTypeConverters();
+    Widget temp;
+    int desiredVisual = -1;
+    XVisualInfo vis;
+    Boolean valid;
+    int depth;
+    int desiredDepth;
+    Colormap colormap;
+    int tempargc;
+    char **tempargv;
+    XtAppContext app_context;
+    Display *display;
+    char phrase[128];
+    int desiredVisualID = -1;	 
 
+    InitTypeConverters();
+	 
+    /*
+     * Create a temp widget that we can interrogate.
+     * This widget will not be realized. 
+     * There should be a better way to do this...
+     */
+    tempargc = argc;
+    tempargv = xmalloc(argc * sizeof(char *));
+    for (i = 0; i < tempargc; i++) {
+        tempargv[i] = xmalloc(strlen(argv[i])+1);
+	strcpy(tempargv[i], argv[i]);
+    }
+    temp = XtAppInitialize(&Global.appContext, "XPaint",
+			   options, XtNumber(options), &tempargc, tempargv,
+			   appDefaults, args, nargs);
+    XtGetApplicationResources(temp, (XtPointer) & appInfo,
+			      resources, XtNumber(resources), NULL, 0);
+    rdb = XtDatabase(XtDisplay(temp));
+    for (i = 0; i < tempargc; i++)
+        free(tempargv[i]);
+	 
+    if (appInfo.help != NULL)
+	usage(argv[0], NULL);
+
+    desiredDepth = 0;
+    if (strcmp(appInfo.visualType, "default") != 0) {
+        char *cp = appInfo.visualType;
+	char *vp = phrase;
+
+	/* this includes the terminating null character */
+	for (i = 0; i <= strlen(cp); i++)
+	    phrase[i] = tolower(cp[i]);
+		 
+	if (strncmp(vp, "cmap", 4) == 0) {
+	    XrmPutStringResource(&rdb, "Canvas*visual", "PseudoColor");
+	    desiredVisual = PseudoColor;
+	    vp += 4;
+	} else if (strncmp(vp, "directcolor", 11) == 0) {
+	    XrmPutStringResource(&rdb, "Canvas*visual", "DirectColor");
+	    desiredVisual = DirectColor;
+	    desiredDepth = 24;
+	    vp += 11;
+	} else if (strncmp(vp, "truecolor", 9) == 0) {
+	    XrmPutStringResource(&rdb, "Canvas*visual", "TrueColor");
+	    desiredVisual = TrueColor;
+	    desiredDepth = 24;
+	    vp += 9;
+	} else if (strncmp(vp, "grayscale", 9) == 0) {
+	    XrmPutStringResource(&rdb, "Canvas*visual", "GrayScale");
+	    desiredVisual = GrayScale;
+	    desiredDepth = 8;
+	    vp += 9;
+	} else if (strncmp(vp, "pseudocolor", 11) == 0) {
+	    XrmPutStringResource(&rdb, "Canvas*visual", "PseudoColor");
+	    desiredVisual = PseudoColor;
+	    vp += 11;
+	} else if (strncmp(vp, "staticgray", 10) == 0) {
+	    XrmPutStringResource(&rdb, "Canvas*visual", "StaticGray");
+	    desiredVisual = StaticGray;
+	    desiredDepth = 8;
+	    vp += 10;
+	} else if (strncmp(vp, "staticcolor", 11) == 0) {
+	    XrmPutStringResource(&rdb, "Canvas*visual", "StaticColor");
+	    desiredVisual = StaticColor;
+	    desiredDepth = 8;
+	    vp += 11;
+	} else if (strncmp(vp, "true", 4) == 0) {
+	    XrmPutStringResource(&rdb, "Canvas*visual", "TrueColor");
+	    desiredVisual = TrueColor;
+	    vp += 4;
+	} else if (strncmp(vp, "gray", 4) == 0) {
+	    XrmPutStringResource(&rdb, "Canvas*visual", "StaticGray");
+	    desiredVisual = StaticGray;
+	    vp += 4;
+	} else if (atoi(vp)>0 || (vp[0]=='0' && vp[1]=='\0')) {
+ 	    desiredVisualID = atoi(vp);
+	    *vp = '\0';
+	} else {
+	    fprintf(stderr, "%s: bad visual type specification '%s'\n",
+		    argv[0], vp);
+	    exit(1);
+	}
+	if (*vp != '\0') {
+	    desiredDepth=atoi(vp);
+	    if (desiredDepth > 0) 
+	        XrmPutStringResource(&rdb, "Canvas*depth", vp);
+	    else
+	        desiredDepth = 0;
+	}
+    }
+    else {
+        /* go find out what the default visual is and get its parameters */
+        Global.vis.depth = DefaultDepth(XtDisplay(temp),
+					DefaultScreen(XtDisplay(temp)));
+	Global.vis.visual = DefaultVisual(XtDisplay(temp),
+					  DefaultScreen(XtDisplay(temp)));
+    }
+
+    if (desiredVisual >= 0 || desiredVisualID >= 0) {
+        vis = GetFutureVisual(&temp, desiredDepth,
+			      desiredVisual, desiredVisualID, &valid);
+		 
+	if (valid) {
+	    depth = vis.depth;
+	    Global.vis = vis;
+	}
+	else {
+	    fprintf(stderr, "No match found for visual\n");
+	    exit(1);
+	}
+    }
+    else {
+        Global.vis.depth = DefaultDepth(XtDisplay(temp),
+					DefaultScreen(XtDisplay(temp)));
+	Global.vis.visual = DefaultVisual(XtDisplay(temp),
+					  DefaultScreen(XtDisplay(temp)));
+    }
+	 
+    sprintf(phrase, "%d", Global.vis.depth);
+    XrmPutStringResource(&rdb, "Canvas*depth", phrase);
+
+    /* destroy our temporary widget */
+    XtDestroyWidget(temp);
+	 
     /*
     **  Create the application context
      */
+
     toplevel = XtAppInitialize(&Global.appContext, "XPaint",
 			       options, XtNumber(options), &argc, argv,
 			       appDefaults, args, nargs);
@@ -223,31 +439,11 @@ main(int argc, char *argv[])
     XtGetApplicationResources(toplevel, (XtPointer) & appInfo,
 			      resources, XtNumber(resources), NULL, 0);
 
-    rdb = XtDatabase(XtDisplay(toplevel));
-
-    if (appInfo.help != NULL)
-	usage(argv[0], NULL);
-
-    if (strcmp(appInfo.visualType, "default") != 0) {
-	char *cp = appInfo.visualType;
-	if (strncmp(cp, "cmap", 4) == 0) {
-	    XrmPutStringResource(&rdb, "Canvas*visual", "PseudoColor");
-	    cp += 4;
-	} else if (strncmp(cp, "true", 4) == 0) {
-	    XrmPutStringResource(&rdb, "Canvas*visual", "TrueColor");
-	    cp += 4;
-	} else if (strncmp(cp, "gray", 4) == 0) {
-	    XrmPutStringResource(&rdb, "Canvas*visual", "StaticGray");
-	    cp += 4;
-	} else {
-	    usage(argv[0], "bad visual type specification");
-	}
-	if (*cp != '\0')
-	    XrmPutStringResource(&rdb, "Canvas*depth", cp);
-    }
+	 
+	 
     if (argc != 1 && argv[1][0] == '-')
 	usage(argv[0], "Invalid option");
-
+	 
     /*
     **  A little initialization
      */
@@ -265,15 +461,16 @@ main(int argc, char *argv[])
     if (isIcon)
 	XrmPutStringResource(&rdb, "Canvas.iconic", "on");
 
+	 
     /*
     **  GRR 960525:  check depth and warn user (use AlertBox() instead?)
      */
     if (!appInfo.nowarn) {
-	int depth;
+	int depth = Global.vis.depth;
 
-	XtVaGetValues(toplevel, XtNdepth, &depth, NULL);
+	/* XtVaGetValues(toplevel, XtNdepth, &depth, NULL); */
 	HelpTextOutput(stderr, "data_loss");
-	fprintf(stderr, "\nYour display depth is %d bits.\n\n", depth);
+	fprintf(stderr, "\nYour canvas depth is %d bits.\n\n", depth);
 	switch (depth) {
 	    case 8:
 	    case 12:
