@@ -2,7 +2,7 @@
  * Miscellaneous definitions and prototypes, including those from misc.c.
  */
 
-/* $Id: misc.h,v 1.18 1996/06/06 10:45:55 torsten Exp $ */
+/* $Id: misc.h,v 1.23 1998/06/09 21:07:47 torsten Exp $ */
 
 #if defined(HAVE_PARAM_H)
 #include <sys/param.h>
@@ -12,17 +12,19 @@
 **  By default everything uses drand48(),
 **    I was making more exceptions than inclusions.
  */
+#if !defined(__EMX__) && !defined(__CYGWIN32__)
 #define USE_DRAND
+#endif
 
 #if defined(SVR4) || defined(__osf__)
 #define SHORT_RANGE
 #else
-#ifndef random
+#if !defined(__GLIBC__) & !defined(random)
 long random(void);
 #endif
 
-#ifndef linux
-#if defined(BSD4_4) || defined(HPArchitecture) || defined(SGIArchitecture) || defined(_AIX)
+#if !defined(linux) & !defined(__EMX__) & !defined(__FreeBSD__)
+#if defined(BSD4_4) || defined(HPArchitecture) || defined(SGIArchitecture) || defined(_AIX) || defined(_SCO_DS)
 void srandom(unsigned int);
 #else
 int srandom(unsigned int);
@@ -50,6 +52,10 @@ extern long lrand48();
 #define SRANDOM(seed)	srandom((unsigned) (seed))
 #endif
 
+#ifdef __EMX__
+#define strcasecmp stricmp
+#endif
+
 /* dialog.c */
 void AlertBox(Widget parent, char *msg, XtCallbackProc okProc,
 	      XtCallbackProc nokProc, void *data);
@@ -118,6 +124,7 @@ double gauss(void);
 int gaussclamp(int range);
 void *xmalloc(size_t n);
 void AutoCrop(Widget paint);
+Widget XtVisCreatePopupShell(String name,  WidgetClass widget_class,  Widget parent, ArgList args, Cardinal num_args);
 
 /* pattern.c */
 void PatternEdit(Widget w, Pixmap pix, Widget button);
