@@ -13,7 +13,7 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: operation.c,v 1.17 1996/11/01 09:37:46 torsten Exp $ */
+/* $Id: operation.c,v 1.19 1998/03/23 21:39:51 torsten Exp $ */
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -43,37 +43,39 @@
 #include "cutCopyPaste.h"
 
 /* Pixmaps for toolbox icons */
-#include "brushOp.xpm"
-#include "eraseOp.xpm"
-#include "sprayOp.xpm"
-#include "pencilOp.xpm"
-#include "dotPenOp.xpm"
-#include "lineOp.xpm"
-#include "arcOp.xpm"
-#include "fillOp.xpm"
-#include "tfillOp.xpm"
+#include "bitmaps/brushOp.xpm"
+#include "bitmaps/eraseOp.xpm"
+#include "bitmaps/sprayOp.xpm"
+#include "bitmaps/pencilOp.xpm"
+#include "bitmaps/dotPenOp.xpm"
+#include "bitmaps/dynPenOp.xpm"
+#include "bitmaps/lineOp.xpm"
+#include "bitmaps/arcOp.xpm"
+#include "bitmaps/fillOp.xpm"
+#include "bitmaps/tfillOp.xpm"
 #ifdef FEATURE_FRACTAL
-#include "ffillOp.xpm"
+#include "bitmaps/ffillOp.xpm"
 #endif
-#include "smearOp.xpm"
-#include "textOp.xpm"
-#include "selectOp.xpm"
-#include "boxOp.xpm"
-#include "rayOp.xpm"
-#include "fboxOp.xpm"
-#include "ovalOp.xpm"
-#include "fovalOp.xpm"
-#include "lassoOp.xpm"
-#include "clineOp.xpm"
-#include "polyOp.xpm"
-#include "fpolyOp.xpm"
-#include "freehandOp.xpm"
-#include "ffreehandOp.xpm"
-#include "selpolyOp.xpm"
+#include "bitmaps/smearOp.xpm"
+#include "bitmaps/textOp.xpm"
+#include "bitmaps/selectOp.xpm"
+#include "bitmaps/boxOp.xpm"
+#include "bitmaps/rayOp.xpm"
+#include "bitmaps/fboxOp.xpm"
+#include "bitmaps/ovalOp.xpm"
+#include "bitmaps/fovalOp.xpm"
+#include "bitmaps/lassoOp.xpm"
+#include "bitmaps/clineOp.xpm"
+#include "bitmaps/polyOp.xpm"
+#include "bitmaps/fpolyOp.xpm"
+#include "bitmaps/freehandOp.xpm"
+#include "bitmaps/ffreehandOp.xpm"
+#include "bitmaps/selpolyOp.xpm"
 
 
 
 static void changeLineAction(Widget, XEvent *);
+static void changeDynPencilAction(Widget, XEvent *);
 static void changeSprayAction(Widget, XEvent *);
 static void changeTFillAction(Widget, XEvent *);
 static void changeBrushAction(Widget, XEvent *);
@@ -197,6 +199,63 @@ brushMenuCallback(Widget w)
     TextPrompt(w, "brushparams", &info, brushOkCallback, NULL, NULL);
 }
 
+/*
+** Set dynamic pencil parameters callbacks
+*/
+static char dynPencilWidthStr[10] = "10";
+static char dynPencilMassStr[10] = "600";
+static char dynPencilDragStr[10] = "15";
+static void dynPencilOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
+{
+    void DynPencilSetParameters(float, float, float);
+    TextPromptInfo *info = (TextPromptInfo*)infoArg;
+    float width = atof(info->prompts[0].rstr);
+    float mass = atof(info->prompts[1].rstr);
+    float drag = atof(info->prompts[2].rstr);
+    
+    if (width < 1 || width > 100) {
+	Notice(w, "Width should be between 1 and 100");
+	return;
+    }
+
+    if (mass < 10 || mass > 2000) {
+	Notice(w, "Mass should be between 10 and 2000");
+	return;
+    }
+
+    if (drag < 0 || drag > 50) {
+	Notice(w, "Drag should be between 0 and 50");
+	return;
+    }
+
+    DynPencilSetParameters(width, mass, drag);
+  
+    sprintf(dynPencilWidthStr, "%0.4g", width);
+    sprintf(dynPencilMassStr, "%0.4g", mass);
+    sprintf(dynPencilDragStr, "%0.4g", drag);
+}
+
+static void dynPencilMenuCallback(Widget w)
+{
+    static TextPromptInfo info;
+    static struct textPromptInfo value[3];
+
+    value[0].prompt = "Width:";
+    value[0].str = dynPencilWidthStr;
+    value[0].len = 5;
+    value[1].prompt = "Mass:";
+    value[1].str = dynPencilMassStr;;
+    value[1].len = 5;
+    value[2].prompt = "Drag:";
+    value[2].str = dynPencilDragStr;;
+    value[2].len = 5;
+    info.prompts = value;
+    info.title = "Enter the desired dynamic pencil parameters";
+    info.nprompt = 3;
+
+    TextPrompt(w, "dynpencilparams", &info, dynPencilOkCallback, NULL, NULL);
+}
+
 /*
 ** Set spray parameters callbacks
  */
@@ -427,6 +486,7 @@ static void tfillPopupCB2(Widget w, XtPointer junk, XtPointer junk2);
 static void ffillPopupCB2(Widget w, XtPointer junk, XtPointer junk2);
 #endif
 static void selectPopupCB(Widget w, XtPointer junk, XtPointer junk2);
+static void dynPencilPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void selectShapePopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void sprayPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void ovalPopupCB(Widget w, XtPointer junk, XtPointer junk2);
@@ -455,6 +515,14 @@ static PaintMenuItem erasePopup[] =
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.erase"),
 };
 
+static PaintMenuItem dynPencilPopup[] = 
+{
+    MI_SEPERATOR(),
+    MI_FLAGCB("autofinish", MF_CHECK, dynPencilPopupCB, 0),
+    MI_SIMPLECB("select", changeDynPencilAction, NULL),
+    MI_SIMPLECB("help", HelpDialog, "toolbox.tools.dynpencil"),
+};
+
 static PaintMenuItem sprayPopup[] =
 {
     MI_SEPERATOR(),
@@ -490,10 +558,10 @@ static PaintMenuItem tfillPopup[] =
     MI_FLAGCB("change", MF_CHECK | MF_GROUP1, fillPopupCB, 1),
     MI_FLAGCB("fill_range", MF_CHECK | MF_GROUP1, fillPopupCB, 2),
     MI_SEPERATOR(),
-MI_FLAGCB("radial", MF_CHECKON | MF_GROUP2, tfillPopupCB2, TFILL_RADIAL),
-  MI_FLAGCB("linear", MF_CHECK | MF_GROUP2, tfillPopupCB2, TFILL_LINEAR),
+    MI_FLAGCB("radial", MF_CHECKON | MF_GROUP2, tfillPopupCB2, TFILL_RADIAL),
+    MI_FLAGCB("linear", MF_CHECK | MF_GROUP2, tfillPopupCB2, TFILL_LINEAR),
     MI_FLAGCB("conical", MF_CHECK | MF_GROUP2, tfillPopupCB2, TFILL_CONE),
-  MI_FLAGCB("square", MF_CHECK | MF_GROUP2, tfillPopupCB2, TFILL_SQUARE),
+    MI_FLAGCB("square", MF_CHECK | MF_GROUP2, tfillPopupCB2, TFILL_SQUARE),
     MI_SIMPLECB("parms", changeTFillAction, NULL),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.tfill"),
 };
@@ -655,6 +723,14 @@ OperationSelectCallAcross(int n)
     MenuCheckItem(selectBPopup[n + 1].widget, True);
     MenuCheckItem(lassoPopup[n + 1].widget, True);
 }
+static void
+dynPencilPopupCB(Widget w, XtPointer junk, XtPointer junk2)
+{
+    Boolean	m = !DynPencilGetFinish();
+
+    DynPencilSetFinish(m);
+    MenuCheckItem(w, m);
+}
 static void 
 sprayPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
@@ -683,6 +759,7 @@ erasePopupCB(Widget w, XtPointer junk, XtPointer junk2)
 
 GENERATE_OP(DotPencil)
 GENERATE_OP(Pencil)
+GENERATE_OP(DynPencil)
 GENERATE_OP(Box)
 GENERATE_OP(FBox)
 GENERATE_OP(Line)
@@ -730,6 +807,8 @@ static IconListItem iconList[] =
      "<BtnDown>(2): changeBrush()", MENU(erase), None},
     {"pencil", XPMMAP(pencil), &PencilOp,
      NULL, MENU(pencil), None},
+    {"dynpencil", XPMMAP(dynPen), &DynPencilOp, 	
+     "<BtnDown>(2): changeDynPencil()", MENU(dynPencil), None},
     {"dotPencil", XPMMAP(dotPen), &DotPencilOp,
      NULL, MENU(dotPencil), None},
     {"selectBox", XPMMAP(select), &SelectBoxOp,
@@ -917,6 +996,11 @@ changeLineAction(Widget w, XEvent * event)
 {
     lineWidth(lineMenu[LW_SELECT].widget, (int) lineMenu[LW_SELECT].data);
 }
+static void
+changeDynPencilAction(Widget w, XEvent *event)
+{
+    dynPencilMenuCallback(w);
+}
 static void 
 changeSprayAction(Widget w, XEvent * event)
 {
@@ -955,6 +1039,7 @@ OperationInit(Widget toplevel)
 	{"changeLine", (XtActionProc) changeLineAction},
 	{"changeBrush", (XtActionProc) changeBrushAction},
 	{"changeFont", (XtActionProc) changeFontAction},
+	{"changeDynPencil", (XtActionProc) changeDynPencilAction},
 	{"changeSpray", (XtActionProc) changeSprayAction},
 	{"changeTFillParms", (XtActionProc) changeTFillAction},
     };
