@@ -10,7 +10,7 @@
 ** implied warranty.
  */
 
-/* $Id: libpnmrw.c,v 1.5 1996/06/03 05:35:25 torsten Exp $ */
+/* $Id: libpnmrw.c,v 1.8 1998/06/09 21:08:57 torsten Exp $ */
 
 #if defined(HAVE_PARAM_H)
 #include <sys/param.h>
@@ -46,6 +46,9 @@
 
 void *malloc(int);
 void free(void *);
+#if defined(__EMX__) || defined(__CYGWIN32__)
+#include <errno.h>
+#endif
 
 
 /* Definitions. */
@@ -131,9 +134,13 @@ pm_perror(char *reason)
 #if defined(BSD4_4)
     __const extern char *__const sys_errlist[];
 #else
+#ifndef __GLIBC__
     extern char *sys_errlist[];
 #endif
+#endif
+#if !defined(__EMX__) || defined(__CYGWIN32__)
     extern int errno;
+#endif
     char *e;
 
     e = sys_errlist[errno];
