@@ -12,7 +12,7 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: writeGIF.c,v 1.8 1996/09/06 06:59:49 torsten Exp $ */
+/* $Id: writeGIF.c,v 1.10 1998/02/24 20:38:21 torsten Exp $ */
 
 /* ppmtogif.c - read a portable pixmap and produce a GIF file
 **
@@ -33,9 +33,16 @@
 ** CompuServe Incorporated.
  */
 
+#ifdef __CYGWIN32__
+#define __EMX__
+#endif
+
 #include <stdio.h>
 #include "image.h"
 #include "rwTable.h"
+#ifdef __EMX__
+#include <errno.h>
+#endif
 
 #define MAXCOLORS 256
 
@@ -77,7 +84,9 @@ static Image *image;
 int
 WriteGIF(char *file, Image * outImage)
 {
+#ifndef __EMX__
     extern int errno;
+#endif
     int r[MAXCOLORS], g[MAXCOLORS], b[MAXCOLORS];
     int i, bpp;
     FILE *fp;
