@@ -12,11 +12,11 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: writePS.c,v 1.3 1996/04/19 09:34:34 torsten Exp $ */
+/* $Id: writePS.c,v 1.4 1998/02/24 20:38:13 torsten Exp $ */
 
 #include <stdio.h>
 #include "image.h"
-#ifdef sco
+#if defined(sco) || defined(__CYGWIN32__)
 #include <time.h>
 #else
 #include <sys/time.h>
